package org.cesar.flip;

import java.io.File;
import java.io.StringReader;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import java_cup.runtime.Symbol;

import org.cesar.flip.flipc.IBuildExecution;
import org.cesar.flip.flipex.ExtractorValidatorMap;
import org.cesar.flip.flipex.RefactoringInfo;
import org.cesar.flip.flipex.extractors.IExtractionTechnique;
import org.cesar.flip.flipex.extractors.IExtractor;
import org.cesar.flip.flipex.validators.IValidator;
import org.cesar.flip.flipex.validators.ValidationResults;
import org.cesar.flip.flipex.validators.ValidatorSet;
import org.cesar.flip.flipex.validators.ValidatorStatus;
import org.cesar.flip.flipg.AbstractComponentFactory;
import org.cesar.flip.flipg.IPLAdapter;
import org.cesar.flip.flipg.model.IComponent;
import org.cesar.flip.flipg.model.IExpression;
import org.cesar.flip.flipg.model.IFeature;
import org.cesar.flip.flipg.model.IFeature.FeatureType;
import org.cesar.flip.flipg.model.expressions.parse.parser;
import org.cesar.flip.flipg.model.expressions.parse.scanner;
import org.cesar.flip.properties.FlipPropertiesConstants;
import org.cesar.flip.util.FlipUtil;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;

/**
 * Default entry point to the services provided by the FLIP platform.
 * 
 * @author Davi Pires (davi.pires@cesar.org.br)
 */
public class Facade {

	/**
	 * The current working project.
	 */
	private IProject workingProject;

	/**
	 * The <code>Facade</code> singleton.
	 */
	private static Facade instance;

	/**
	 * The set where the validators existing in the system are stored.
	 */
	private ValidatorSet validatorSet;

	/**
	 * Stores a map between which validators are required by installed
	 * extractors. (Extractor -> {Validator1, Validator2, ...})
	 */
	private ExtractorValidatorMap extractorValidatorMap;

	/**
	 * All installed Product Line Adapters.
	 */
	private List<IPLAdapter> installedPLAdapters;

	/**
	 * The Product Line Adapter that will be used for all services.
	 */
	private IPLAdapter plAdapter;

	/**
	 * The path to the feature model file.
	 */
	private String featureModelPath;

	/**
	 * The path to the configuration knowledge file.
	 */
	private String configurationKnowledgePath;

	/**
	 * The list of recent transactions.
	 */
	private List<Transaction> transactions;
	
	/**
	 * Map between the ID of the technique
	 * and the technique.
	 */
	private Map<String, IExtractionTechnique> techniques;

	/**
	 * Number of transactions to be storage.
	 */
	private int transactionNumber;

	/**
	 * List of features 
	 */
	private List<IFeature> features;

	/**
	 * Factory of components
	 */
	private AbstractComponentFactory componentFactory;

	/**
	 * build execution module
	 */
	private IBuildExecution buildExecution;

	/**
	 * Creates a new instance of <code>Facade</code>.
	 * 
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.PLADAPTER_UNAVAILABLE</code>
	 *             When there's no installed PL Adapter.</li>
	 *             </ul>
	 * 
	 * @see FacadeStarter
	 * @see Facade#getInstance()
	 */
	private Facade() throws CoreException {
		this.validatorSet = new ValidatorSet();
		this.extractorValidatorMap = new ExtractorValidatorMap();
		this.installedPLAdapters = new LinkedList<IPLAdapter>();
		this.techniques = new HashMap<String, IExtractionTechnique>();
		this.componentFactory = new AbstractComponentFactory();

		FacadeStarter.start(this);

		if (this.installedPLAdapters.isEmpty()) {
			throw new CoreException(new Status(IStatus.ERROR,
					FlipPlugin.PLUGIN_ID,
					ExceptionConstants.PLADAPTER_UNAVAILABLE,
					FlipMessages.Facade_NO_PL_ADAPTER_INSTALLED, null));
		}

		this.setPlAdapter(this.installedPLAdapters.get(0));
		this.transactions = new LinkedList<Transaction>();
	}

	/**
	 * Returns the <code>Facade</code> Singleton.<br>
	 * <br>
	 * When the <code>Facade</code> is first instantiated, it searches the
	 * environment for installed extractors and validators and registers them
	 * and their dependencies.
	 * 
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.PLADAPTER_UNAVAILABLE</code>
	 *             When there's no installed PL Adapter.</li>
	 *             </ul>
	 */
	public static Facade getInstance() throws CoreException {
		if (Facade.instance == null) {
			try {
				Facade.instance = new Facade();
			} catch (CoreException e) {
				FlipPlugin.getDefault().getLog().log(e.getStatus());
				throw e;
			}
		}
		return Facade.instance;
	}

	/**
	 * Checks which extractions are possible given the set of installed
	 * validators and the refactoring described by the
	 * <code>RefactoringInfo</code> object passed as a parameter. <br>
	 * <br>
	 * Returns a <code>ValidationResults</code> object, which is basically a
	 * map between each extractor and the problems that ocurred during the
	 * validation of its required validators.
	 * 
	 * @param info
	 *            the information about the requested extraction
	 * @return a map between each extractor and the validation problems its
	 *         validators reported
	 * 
	 * @see org.cesar.flip.flipex.validators.ValidatorSet#getValidatorsStatusOrigin(RefactoringInfo)
	 * @see org.cesar.flip.flipex.ExtractorValidatorMap#getExtractorStatus(List)
	 * @see org.cesar.flip.flipex.validators.ValidationResults
	 */
	public ValidationResults getValidatorsStatusOrigin(RefactoringInfo info) {
		List<ValidatorStatus> status = this.validatorSet
				.getValidatorsStatusOrigin(info);
		ValidationResults results = this.extractorValidatorMap
				.getExtractorStatus(status);
		return results;
	}

	/**
	 * The same as {@link #getValidatorsStatusOrigin(RefactoringInfo)}, except
	 * that it validates at the destination side, not the origin side.
	 * 
	 * @param info
	 *            the information about the requested extraction
	 * @param extractor
	 *            the selected extractor
	 * 
	 * @return a map between each extractor and the validation problems its
	 *         validators reported
	 * 
	 * @see #getValidatorsStatusOrigin(RefactoringInfo)
	 * @see org.cesar.flip.flipex.validators.ValidatorSet#getValidatorsStatusOrigin(RefactoringInfo)
	 * @see org.cesar.flip.flipex.ExtractorValidatorMap#getExtractorStatus(List)
	 * @see org.cesar.flip.flipex.validators.ValidationResults
	 */
	public ValidationResults getValidatorsStatusDestination(
			RefactoringInfo info, IExtractor extractor) {
		List<IValidator> validators = this.extractorValidatorMap
				.getValidators(extractor);

		List<ValidatorStatus> status = this.validatorSet
				.getValidatorsStatusDestination(info, validators);
		ValidationResults results = this.extractorValidatorMap
				.getExtractorStatus(status);

		return results;
	}

	/**
	 * Runs the provided extraction.<BR>
	 * <BR>
	 * 
	 * The extraction runs in three steps. First, it calls the
	 * <code>init(RefactoringInfo)</code> method of the extractor, passing the
	 * provided <code>RefactoringInfo</code> as an argument. Then, it calls
	 * the <code>extract()</code> method, and lastly the <code>clear()</code>
	 * method.
	 * 
	 * @param extractor
	 *            the extractor to be run.
	 * @param info
	 *            description of the refactoring.
	 * @throws CoreException
	 */
	public void runExtraction(IExtractor extractor, RefactoringInfo info)
			throws CoreException {
		extractor.init(info);
		extractor.extract();
		extractor.clear();
	}

	/**
	 * Returns the mapping between extractors and their required validators.
	 * 
	 * @return the mapping between extractors and validators
	 */
	ExtractorValidatorMap getValidatorExtractorMap() {
		return this.extractorValidatorMap;
	}

	/**
	 * Returns the list of installed validators.
	 * 
	 * @return the list of installed validators.
	 */
	ValidatorSet getValidatorSet() {
		return this.validatorSet;
	}

	/**
	 * Returns the list of installed <code>IPLAdapter</code>.
	 * 
	 * @return the list of installed product line adapters
	 */
	public List<IPLAdapter> getInstalledPLAdapters() {
		return this.installedPLAdapters;
	}

	/**
	 * Return the features from the feature model.
	 * 
	 * @return an unmodifiable list of features.
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             </ul>
	 */
	public List<IFeature> getFeatures() throws CoreException {
		if (this.features == null) {
			this.features = this.plAdapter.getFeatures(this.featureModelPath);
		}
		return this.features;
	}

	/**
	 * Return the components from the component model.
	 * 
	 * @param componentType Type of the components to be retrieve.
	 * @return an unmodifiable list of components.
	 * @throws CoreException
	 * 			   with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             </ul>
	 */
	public List<IComponent> getComponents(String componentType)
			throws CoreException {
		return this.plAdapter.getComponents(this.configurationKnowledgePath,
				componentType);
	}

	/**
	 * Updates the feature model with the changes done in the extraction.
	 * Features can be updated or created.
	 * 
	 * @param features
	 *            features to be updated or created
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             <li><code>ExceptionConstants.CANT_UPDATE_MODELS</code>
	 *             when an error occurs while updating the model.</li>
	 *             </ul>
	 */
	public void updateFeatureModel(List<IFeature> features)
			throws CoreException {
		this.plAdapter.updateFeatureModel(features, this.featureModelPath);
	}

	/**
	 * Sets the Product Line Adapter that is currently in use.
	 * 
	 * @param plAdapter
	 *            the Product Line Adapter
	 */
	public void setPlAdapter(IPLAdapter plAdapter) {
		this.plAdapter = plAdapter;
	}

	/**
	 * Gets the path to the configuration knowledge.
	 * 
	 * @return The path to the configuration knowledge.
	 */
	public String getConfigurationKnowledgePath() {
		return this.configurationKnowledgePath;
	}

	/**
	 * Gets the path to the feature model.
	 * 
	 * @return The path to the feature model
	 */
	public String getFeatureModelPath() {
		return this.featureModelPath;
	}

	/**
	 * Updates the configuration knowledge, creating new components or updating
	 * existing ones.
	 * 
	 * @param components
	 *            the components to be added or updated
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             <li><code>ExceptionConstants.CANT_UPDATE_MODELS</code>
	 *             when an error occurs while updating the model.</li>
	 *             </ul>
	 */
	public void updateConfigurationKnowledge(List<IComponent> components)
			throws CoreException {
		this.plAdapter.updateConfigurationKnowledge(components,
				this.configurationKnowledgePath);
	}

	/**
	 * Saves the changes made to all models.
	 * 
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_SAVE_MODELS</code> when
	 *             there was an error saving the models.</li>
	 *             </ul>
	 */
	public void saveModels() throws CoreException {
		this.plAdapter.saveModels();
	}

	/**
	 * Returns the current working project.
	 * 
	 * @return the current working project.
	 */
	public IProject getWorkingProject() {
		return this.workingProject;
	}

	/**
	 * Sets the current working project.
	 * 
	 * @param workingProject
	 *            the new working project
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.RESOURCE_DOES_NOT_EXIST</code>
	 *             when the feature model and configuration knowledge properties
	 *             don't exist.</li>
	 *             </ul>
	 */
	public void setWorkingProject(IProject workingProject) throws CoreException {
		IProjectNature nature = workingProject.getNature(FlipNature.NATURE_ID);
		if(nature instanceof FlipNature){
			this.workingProject = workingProject;
			try {
				this.featureModelPath = workingProject
						.getPersistentProperty(FlipPropertiesConstants.PROJECT_FEATURE_MODEL_LOCATION);
				this.configurationKnowledgePath = workingProject
						.getPersistentProperty(FlipPropertiesConstants.PROJECT_CONFIGURATION_KNOWLEDGE_LOCATION);
			} catch (CoreException e) {
				IStatus status = new Status(IStatus.ERROR, FlipPlugin.PLUGIN_ID,
						ExceptionConstants.RESOURCE_DOES_NOT_EXIST,
						FlipMessages.Facade_RESOURCE_DOES_NOT_EXIST, e);
				throw new CoreException(status);
			}
		} else {
			IStatus status = new Status(IStatus.ERROR, FlipPlugin.PLUGIN_ID,
					ExceptionConstants.SELECTED_PROJECT_IS_NOT_FLIP_PROJECT,
					FlipMessages.Facade_IS_NOT_A_FLIP_PROJECT, null);
			throw new CoreException(status);
		}

	}

	/**
	 * Closes all open models.
	 * 
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_CLOSE_MODELS</code> when
	 *             there was an error closing the models.</li>
	 *             </ul>
	 */
	public void closeModels() throws CoreException {
		try {
			this.features = null;
			this.plAdapter.closeModels();
		} catch (CoreException e) {
			FlipPlugin.getDefault().getLog().log(e.getStatus());
			throw e;
		}
	}

	/**
	 * Gets a <code>List</code> of transactions used by the user previously.
	 * 
	 * @return A <code>List</code> of transactions
	 */
	public List<Transaction> getTransactions() {
		return this.transactions;
	}

	/**
	 * Saves a new <code>Transaction</code>.
	 * 
	 * @param transaction
	 *            the transaction to be saved
	 */
	public void saveTransaction(Transaction transaction) {
		this.transactions.add(0, transaction);

	}

	/**
	 * Creates a feature object for storage in the feature model.
	 * 
	 * @param name
	 *            the name of the feature
	 * @param featureType
	 *            the feature's type
	 * @param parent
	 *            the parent feature
	 * @return the feature object
	 */
	public IFeature createFeature(String name, FeatureType featureType,
			IFeature parent) {
		return this.plAdapter.createFeature(name, featureType, parent);
	}

	/**
	 * Performs the steps necessary to generate the instances of the product
	 * line, the work is mostly delegated to the Product Line Adapter.
	 * 
	 * @param project
	 *            the project to be built
	 * @param monitor
	 *            the progress monitor
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.NO_INSTANCE_DESCRIPTORS_FOUND</code>
	 *             when there were no instance descriptors found in the project.</li>
	 *             </ul>
	 */
	public void transformAllModels(IProject project, IProgressMonitor monitor)
			throws CoreException {
		List<String> modelDescriptorsPaths = this.plAdapter.acquireModelsFromProject(project);
		if (modelDescriptorsPaths.isEmpty()) {
			IStatus status = new Status(IStatus.ERROR, FlipPlugin.PLUGIN_ID,
					ExceptionConstants.NO_INSTANCE_DESCRIPTORS_FOUND,
					FlipMessages.Facade_ERROR_NO_PRODUCT_INSTANCE_DESCRIPTORS
							+ project.getName(), null);
			throw new CoreException(status);
		}
		if (monitor != null) {
			monitor.beginTask(FlipMessages.Facade_TRANSFORMING_MODELS,
					modelDescriptorsPaths.size());
		}
		String transformationOutput = project
				.getPersistentProperty(FlipPropertiesConstants.TRANSFORMATION_OUTPUT_PATH_LOCATION);
		if ((transformationOutput != null) && !transformationOutput.equals("")) {
			this.clearOutputPath(transformationOutput);
			Path outputPath = new Path(transformationOutput);
			this.buildExecution.transformAllModels(project, modelDescriptorsPaths,
					outputPath, monitor);
		} else {
			IStatus status = new Status(
					IStatus.ERROR,
					FlipPlugin.PLUGIN_ID,
					ExceptionConstants.TRANSFORMATION_OUTPUT_PATH_PROPERTY_NOT_SET,
					FlipMessages.Facade_ERROR_TRANSFORMATION_OUTPUT_PATH_NOT_SET
							+ project.getName(), null);
			throw new CoreException(status);
		}
	}

	/**
	 * Clear the output path
	 * 
	 * @param path 
	 * 			output root path. 			
	 */
	private void clearOutputPath(String path) {
		File output = new File(path);
		FlipUtil.deleteDirectory(output);
		output.mkdir();
	}

	/**
	 * Returns the project nature of the variability management tool.
	 * 
	 * @return the project nature of the variability management tool
	 */
	public String getProductLineNature() {
		return this.plAdapter.getProductLineNature();
	}

	/**
	 * Copies the default files to the project.
	 * 
	 * @param project
	 *            the project where the files will be copied to
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.ERROR_COPYING_FILES_TO_PROJECT</code>
	 *             when there was an error copying the default files to a new
	 *             project.</li>
	 *             </ul>
	 */
	public void copyFilesToNewProject(IProject project) throws CoreException {
		this.plAdapter.copyFilesToNewProject(project);
	}

	/**
	 * Gets the number of transactions.
	 * @return
	 */
	public int getTransactionNumber() {
		return this.transactionNumber;
	}

	/**
	 * Sets the transaction number
	 * 
	 * @param transactionNumber
	 * @throws CoreException
	 * 			   with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.INVALID_TRANSACTIONS_NUMBER</code>
	 *             when there was an error setting a wrong value.</li>
	 *             </ul>
	 */
	public void setTransactionNumber(int transactionNumber)
			throws CoreException {
		if (transactionNumber > 0) {
			this.transactionNumber = transactionNumber;
		} else {
			IStatus status = new Status(IStatus.ERROR, FlipPlugin.PLUGIN_ID,
					ExceptionConstants.INVALID_TRANSACTIONS_NUMBER,
					FlipMessages.Facade_INVALID_NUMBER_OF_TRANSACTIONS, null);
			throw new CoreException(status);
		}
	}

	/**
	 * Creates a new component with the given type.
	 * 
	 * @param componentType
	 * @return
	 * 		A new Component
	 */
	public IComponent createComponent(String componentType) {
		return this.componentFactory.createComponent(componentType);
	}

	/**
	 * Gets a set of extractors.
	 * @return
	 */
	public Set<IExtractor> getExtractors() {
		return this.extractorValidatorMap.getExtractors();
	}

	/**
	 * Parses the given text and verifies if is valid.
	 * 
	 * @param text
	 * @return
	 * @throws CoreException
	 */
	public IExpression parseExpression(String text) throws CoreException {
		IExpression exp = null;
		scanner s = new scanner(new StringReader(text));
		parser p = new parser(s, this.features);
		try {
			Symbol sym = p.parse();
			exp = (IExpression) sym.value;
		} catch (Exception e) {
			if (e instanceof CoreException) {
				throw (CoreException) e;
			}
			IStatus status = new Status(IStatus.ERROR, FlipPlugin.PLUGIN_ID,
					FlipMessages.Facade_INVALID_EXPRESSION);
			throw new CoreException(status);
		} catch (Error e) {
			IStatus status = new Status(IStatus.ERROR, FlipPlugin.PLUGIN_ID,
					FlipMessages.Facade_INVALID_CHARACTER + (s.yycolumn() + 1)
							+ ": " + "'" + text.charAt(s.yycolumn()) + "'");
			throw new CoreException(status);
		}
		return exp;
	}

	/**
	 * Adds a new Technique registered as a extension point.
	 * 
	 * @param id
	 * 			id of the technique
	 * @param technique 			 
	 */
	public void addTechnique(String id, IExtractionTechnique technique) {
		this.techniques.put(id, technique);
	}

	/**
	 * Adds a extractor registered as a extension point.
	 * 
	 * @param techniqueID 
	 * 				technique associated with this extractor.
	 * @param newExtractor
	 */
	public void addExtractor(String techniqueID, IExtractor newExtractor) {
		this.extractorValidatorMap.registerExtractor(newExtractor);
		IExtractionTechnique technique = this.techniques.get(techniqueID);
		technique.addExtractor(newExtractor);
		newExtractor.setExtractionTechnique(technique);
	}

	/**
	 * Gets the techniques registered in the FLiP plugin.
	 * @return
	 */
	public Map<String, IExtractionTechnique> getTechniques() {
		return this.techniques;
	}

	/**
	 * Gets the abstract component factory
	 * @return
	 */
	public AbstractComponentFactory getAbstractComponentFactory() {
		return this.componentFactory;
	}

	/**
	 * Gets the installed build execution module
	 * @return
	 */
	public IBuildExecution getInstalledBuildExecution() {
		return this.buildExecution;
	}
	
	/**
	 * Sets the installed build execution module
	 * @param buildExecution
	 * @return
	 */
	public IBuildExecution setInstalledBuildExecution(IBuildExecution buildExecution) {
		return this.buildExecution = buildExecution;
	}

	/**
	 * Runs the build at the specified project.
	 * 
	 * @param project
	 * @param monitor
	 * @return
	 * @throws CoreException
	 */
	public String runBuild(IProject project, IProgressMonitor monitor) throws CoreException{
		return this.buildExecution.runBuild(project,
				monitor);
	}
}
