package org.cesar.flip;

import org.cesar.flip.flipc.IBuildExecution;
import org.cesar.flip.flipex.extractors.IExtractionTechnique;
import org.cesar.flip.flipex.extractors.IExtractor;
import org.cesar.flip.flipex.validators.IValidator;
import org.cesar.flip.flipg.IPLAdapter;
import org.cesar.flip.flipg.model.IComponentFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;

/**
 * Searches the platform (installed plugins) for Extractors and Validators, and
 * registers them in the provided <code>Facade</code>.
 * 
 * @author Davi Pires (davi.pires@cesar.org.br)
 * 
 */
public class FacadeStarter {
	
	/**
	 * A <code>FacadeStarter</code> instance.
	 */
	private static FacadeStarter starter;

	/**
	 * The <code>Facade</code> where the references to the extractors and
	 * validators will be stored.
	 */
	private static Facade facade;

	/**
	 * Registers installed validators and extractors.
	 */
	public static void start(Facade target) {
		FacadeStarter.starter = new FacadeStarter();
		FacadeStarter.facade = target;
		FacadeStarter.traverseExtensions("org.cesar.flip.extractionTechnique",
				FacadeStarter.starter.new TechniqueProcessor());
		FacadeStarter.traverseExtensions("org.cesar.flip.validatorSets",
				FacadeStarter.starter.new ValidatorProcessor());
		FacadeStarter.traverseExtensions("org.cesar.flip.extractorSets",
				FacadeStarter.starter.new ExtractorProcessor());
		FacadeStarter.traverseExtensions("org.cesar.flip.productLineAdapter",
				FacadeStarter.starter.new PLAdapterProcessor());
		FacadeStarter.traverseExtensions("org.cesar.flip.componentFactory",
				FacadeStarter.starter.new IComponentFactoryProcessor());
		FacadeStarter.traverseExtensions("org.cesar.flip.buildExecution",
				FacadeStarter.starter.new BuildExecutionProcessor());
	}

	/**
	 * Gets the extensions points based on the extension point name.
	 * 
	 * @param extensionPoint
	 * 					extension point name
	 * @param processor
	 */
	private static void traverseExtensions(String extensionPoint,
			IConfigurationElementProcessor processor) {
		IExtensionPoint extensionPoints = Platform.getExtensionRegistry()
				.getExtensionPoint(extensionPoint);
		IExtension[] installedExtensions = extensionPoints.getExtensions();
		FacadeStarter
				.visitConfigurationElements(installedExtensions, processor);
	}

	/**
	 * Visits installed plugins.<BR>
	 * 
	 * Visit-style method that sends an <code>IConfigurationElementProcessor
	 * </code>
	 * through a list of extensions, calling its <code>process</code> method
	 * on each <code>IConfigurationElement</code> registered by them.
	 */
	private static void visitConfigurationElements(IExtension[] extensions,
			IConfigurationElementProcessor processor) {
		for (IExtension extension : extensions) {
			IConfigurationElement[] configElements = extension
					.getConfigurationElements();
			for (IConfigurationElement configElement : configElements) {
				try {
					processor.process(configElement);
				} catch (CoreException e) {
					Status status = new Status(IStatus.ERROR, configElement
							.getContributor().getName(), IStatus.OK, e
							.getMessage(), e.getCause());
					FlipPlugin.getDefault().getLog().log(status);
				}
			}
		}
	}

	/**
	 * Default interface for the visitor objects that will process the
	 * <code>IConfigurationElement</code>s registered in the platform.
	 */
	private interface IConfigurationElementProcessor {
		/**
		 * @param element
		 *            The configuration element that is going to be processed.
		 * @throws CoreException
		 *             When there was an error loading a class specified in an
		 *             extension
		 */
		public void process(IConfigurationElement element) throws CoreException;
	}

	/**
	 * Visitor for the <code>org.cesar.flip.validatorSets</code> extensions.<BR>
	 * Registers all validators in the environment in the validator set.
	 */
	private class ValidatorProcessor implements IConfigurationElementProcessor {

		/**
		 * @see org.cesar.flip.FacadeStarter.IConfigurationElementProcessor#process(org.eclipse.core.runtime.IConfigurationElement)
		 */
		public void process(IConfigurationElement element) throws CoreException {
			IValidator validatorObject = (IValidator) element
					.createExecutableExtension("class");

			FacadeStarter.facade.getValidatorSet()
					.addValidator(validatorObject);
		}
	}

	/**
	 * Visitor for the <code>org.cesar.flip.extractionTechnique</code>
	 * extensions.<BR>
	 * Registers all techniques in the environment.
	 */
	private class TechniqueProcessor implements IConfigurationElementProcessor {

		/**
		 * @see org.cesar.flip.FacadeStarter.IConfigurationElementProcessor#process(org.eclipse.core.runtime.IConfigurationElement)
		 */
		public void process(IConfigurationElement element) throws CoreException {
			IExtractionTechnique technique = (IExtractionTechnique) element
					.createExecutableExtension("class");
			technique.setDescription(element.getAttribute("description"));
			technique.setName(element.getAttribute("name"));
			FacadeStarter.facade.addTechnique(element.getAttribute("class"),
					technique);
		}
	}

	/**
	 * Visitor for the <code>org.cesar.flip.ProductLineAdapter</code>
	 * extensions.<BR>
	 * Registers all Product Line Adapters in the environment.
	 */
	private class PLAdapterProcessor implements IConfigurationElementProcessor {

		/**
		 * @see org.cesar.flip.FacadeStarter.IConfigurationElementProcessor#process(org.eclipse.core.runtime.IConfigurationElement)
		 */
		public void process(IConfigurationElement element) throws CoreException {
			IPLAdapter adapterObject = (IPLAdapter) element
					.createExecutableExtension("class");

			FacadeStarter.facade.getInstalledPLAdapters().add(adapterObject);
		}
	}

	/**
	 * Visitor for the <code>org.cesar.flip.ProductLineAdapter</code>
	 * extensions.<BR>
	 * Registers all Product Line Adapters in the environment.
	 */
	private class IComponentFactoryProcessor implements
			IConfigurationElementProcessor {

		/**
		 * @see org.cesar.flip.FacadeStarter.IConfigurationElementProcessor#process(org.eclipse.core.runtime.IConfigurationElement)
		 */
		public void process(IConfigurationElement element) throws CoreException {
			IComponentFactory factory = (IComponentFactory) element
					.createExecutableExtension("class");

			FacadeStarter.facade.getAbstractComponentFactory().registerFactory(
					factory);
		}
	}

	/**
	 * Visitor for the <code>org.cesar.flip.extractorSets</code> extensions.<BR>
	 * Registers all validators in the environment in the validator set.
	 * 
	 */
	private class ExtractorProcessor implements IConfigurationElementProcessor {

		/**
		 * @see org.cesar.flip.FacadeStarter.IConfigurationElementProcessor#process(org.eclipse.core.runtime.IConfigurationElement)
		 */
		public void process(IConfigurationElement configElement)
				throws CoreException {
			IExtractor newExtractor = (IExtractor) configElement
					.createExecutableExtension("class");
			newExtractor.setName(configElement.getAttribute("name"));
			newExtractor.setDescription(configElement
					.getAttribute("description"));
			String techniqueID = configElement
					.getAttribute("extractionTechniqueClass");

			FacadeStarter.facade.addExtractor(techniqueID, newExtractor);

			for (IConfigurationElement configurationElement : configElement
					.getChildren("requiredValidators")) {
				this.processRequiredValidators(configurationElement,
						newExtractor);
			}
		}

		/**
		 * Gets the required validators for the extractor.
		 * 
		 * @param requiredValidators
		 * @param extractor
		 */
		private void processRequiredValidators(
				IConfigurationElement requiredValidators, IExtractor extractor) {
			for (IConfigurationElement requiredValidator : requiredValidators
					.getChildren()) {
				String validatorClassName = requiredValidator
						.getAttribute("class");

				IValidator validator = FacadeStarter.facade.getValidatorSet()
						.getValidator(validatorClassName);
				FacadeStarter.facade.getValidatorExtractorMap().put(extractor,
						validator);
			}
		}
	}
	
	/**
	 * Visitor for the <code>org.cesar.flip.ProductLineAdapter</code>
	 * extensions.<BR>
	 * Registers all Product Line Adapters in the environment.
	 */
	private class BuildExecutionProcessor implements
			IConfigurationElementProcessor {

		/**
		 * @see org.cesar.flip.FacadeStarter.IConfigurationElementProcessor#process(org.eclipse.core.runtime.IConfigurationElement)
		 */
		public void process(IConfigurationElement element) throws CoreException {
			IBuildExecution build = (IBuildExecution) element
					.createExecutableExtension("class");

			FacadeStarter.facade.setInstalledBuildExecution(build);
		}
	}
}