package org.cesar.flip;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ISaveParticipant;
import org.eclipse.core.resources.ISavedState;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;


/**
 * Controls the plug-in life cycle. It is useful to keep a reference to this
 * class, since through it we get access to information about the environment,
 * and the context of the execution of the plugin.
 *
 * @author Davi Pires (davi.pires@cesar.org.br)
 */
public class FlipPlugin extends AbstractUIPlugin {

	/**
	 * The plug-in ID.
	 */
	public static final String PLUGIN_ID = "org.cesar.flip";

	/**
	 * Shared instance of the FlipPlugin.
	 */
	private static FlipPlugin plugin;

	/**
	 * Creates a new instance of this class, and stores it. This instance can be
	 * retrieved by the <code>getDefault()</code> method.
	 *
	 * @see org.cesar.flip.FlipPlugin#getDefault()
	 */
	public FlipPlugin() {
		FlipPlugin.plugin = this;
	}

	/**
	 * Called when the plugin is started by the platform.
	 *
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		ISaveParticipant saveParticipant = new FlipPluginSaveParticipant();
		ISavedState lastState = ResourcesPlugin.getWorkspace()
				.addSaveParticipant(this, saveParticipant);
		if (lastState == null)
			return;
		IPath location = lastState.lookup(new Path(FlipMessages.flip_plugin_path));
		if (location == null)
			return;
		// the plugin instance should read any important state from the file.
		File f = getStateLocation().append(location).toFile();
		readStateFrom(f);
	}

	/**
	 * Reads the last state of the plugin from file. 
	 * 
	 * @param target
	 * 				file to be read.
	 */
	protected void readStateFrom(File target) {
		try {
			if (target.exists()) {

				BufferedReader br = new BufferedReader(new FileReader(target));				
				
				String read = br.readLine();
				while (!read.equals("##")) {
					Map<String, String> associationsMap = new HashMap<String, String>();
					String name = read;
					String temp1 = br.readLine();
					while(!temp1.endsWith("%")){
						String feature = temp1;
						String component = br.readLine();
						associationsMap.put(feature, component);
						temp1 = br.readLine();
					}

					String projectName = br.readLine();
					IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);

					List<IFile> destinationFiles = new LinkedList<IFile>();
					String destFile = br.readLine();
					while (!destFile.equals("#")) {
						IFile file = project.getFile(destFile);
						destinationFiles.add(file);
						destFile = br.readLine();
					}

					Transaction transaction = new Transaction(name, associationsMap,
							destinationFiles);
					Facade.getInstance().saveTransaction(transaction);
					read = br.readLine();
				}

			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Writes the actual state of the plugin to file. 
	 * 
	 * @param target
	 * 				file to be write.
	 */
	protected void writeImportantState(File target) {

		try {
			
			int transactionNumber = Facade.getInstance().getTransactionNumber();
				
			if (!target.exists()) {
				target.createNewFile();
			}

			List<Transaction> transactions;
			transactions = Facade.getInstance().getTransactions();
			String transactionText = "";
			int numberOfTransactions = 0;
			
			Collections.reverse(transactions);			            
			for (Transaction transaction: transactions) {
				if (numberOfTransactions == transactionNumber) {break;}
				transactionText += createTransactionText(transaction);
				numberOfTransactions++;
			}
			transactionText += "##";
			PrintWriter pw = new PrintWriter(new FileWriter(target));
			pw.print(transactionText);
			pw.close();

	    } catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * Creates the transactions text to be save with the 
	 * plugin state.
	 * 
	 * @param transaction
	 */
	private String createTransactionText(Transaction transaction) {
		String ret = "";

		ret += transaction.getName() + "\n";
		
		HashMap<String, String> associations = (HashMap<String, String>)transaction.getAssociations();
		Iterator<String> it = associations.keySet().iterator();
		while(it.hasNext()){
			String feature = it.next();
			String aspect = associations.get(feature);
			ret += feature + "\n";
			ret += aspect+ "\n";
		}	
		
		ret += "%\n";
		
		ret += transaction.getProject().getName() + "\n";
		for (IFile file: transaction.getDestinations()) {
			ret += file.getProjectRelativePath() + "\n";
		}
		ret += "#\n";
		return ret;
	}

	/**
	 * Called when the plugin is stopped by the platform.
	 *
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		FlipPlugin.plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance of the FlipPlugin class.
	 *
	 * @return the latest instance of this class created with the default
	 *         constructor.
	 */
	public static FlipPlugin getDefault() {
		return FlipPlugin.plugin;
	}

}
