package org.cesar.flip.flipex;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.cesar.flip.Facade;
import org.cesar.flip.flipex.extractors.IExtractor;
import org.cesar.flip.flipex.validators.IValidator;
import org.cesar.flip.flipex.validators.ValidationResults;
import org.cesar.flip.flipex.validators.ValidatorStatus;

/**
 * A map between <code>IExtractor</code>s and <code>IValidator</code>s.
 * 
 * @author Davi Pires (davi.pires@cesar.org.br)
 * 
 */
public class ExtractorValidatorMap {

	/**
	 * A mapping betwen extractors and the validators they require.
	 */
	private Map<IExtractor, List<IValidator>> map;

	/**
	 * Returns a list with the validators required for the provided extractor;
	 * 
	 * @param extractor
	 *            The extractor whose validators will be returned
	 * @return The validators required for the provided extrator
	 */
	public List<IValidator> getValidators(IExtractor extractor) {
		return this.map.get(extractor);
	}

	/**
	 * Creates a new empty instance of <code>ExtractorValidatorMap</code>.
	 */
	public ExtractorValidatorMap() {
		this.map = new HashMap<IExtractor, List<IValidator>>();
	}

	/**
	 * Adds a new mapping between the given extractor and validator.
	 * 
	 * @param extractor
	 * @param validator
	 */
	public void put(IExtractor extractor, IValidator validator) {
		if (validator == null) {
			return;
		}

		if (this.map.containsKey(extractor)) {
			List<IValidator> validators = this.map.get(extractor);
			validators.add(validator);
		} else {
			List<IValidator> validators = new LinkedList<IValidator>();
			validators.add(validator);
			this.map.put(extractor, validators);
		}
	}

	/**
	 * Register a new extractor
	 * 
	 * @param newExtractor
	 */
	public void registerExtractor(IExtractor newExtractor) {
		this.map.put(newExtractor, new LinkedList<IValidator>());
	}

	/**
	 * Checks if the given validator is required by <code>extractor</code>.
	 * 
	 * @param extractor
	 *            the extractor to be checked
	 * @param validator
	 *            the validator to be checked
	 * @return <b><code>true</code></b> if <code>validator</code> is
	 *         required by <code>extractor</code>, and <b><code>false</code></b>
	 *         otherwise.
	 */
	public boolean isValidatorOf(IExtractor extractor, IValidator validator) {
		return this.map.containsKey(extractor)
				&& this.map.get(extractor).contains(validator);

	}

	/**
	 * Gives a list of valid extractors, based on the status of the validators.
	 * 
	 * @param status
	 *            a list of the status of each validator in the system.
	 * @return a <code>ValidationResults</code> object.
	 * 
	 * @see Facade#getValidatorsStatusOrigin(org.cesar.flip.flipex.RefactoringInfo)
	 * @see ValidationResults
	 */
	public ValidationResults getExtractorStatus(List<ValidatorStatus> status) {
		ValidationResults results = new ValidationResults();

		Set<IExtractor> extractors = this.map.keySet();
		LinkedHashSet<IExtractor> orderedExtractors = new LinkedHashSet<IExtractor>(
				extractors);
		for (IExtractor extractor : orderedExtractors) {
			results.addExtractor(extractor);
			for (ValidatorStatus validatorStatus : status) {
				if (this.isValidatorOf(extractor, validatorStatus.getParent())
						&& !validatorStatus.isSuccessful()) {
					results.addFailedValidator(extractor, validatorStatus);
				}
			}
		}

		return results;
	}

	/**
	 * Gets the extractors set.
	 * 
	 * @return 
	 * 		the extractors	
	 */
	public Set<IExtractor> getExtractors() {
		return this.map.keySet();
	}

}
