package org.cesar.flip.flipex;

import java.io.IOException;
import java.util.LinkedList;

import org.cesar.flip.flipex.util.PreprocessingUtil;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;

/**
 * Represents a preprocessed Java selection for refactoring.
 * 
 * @author Vilmar Nepomuceno (vilmar.nepomuceno@cesar.org.br)
 * 
 */
public class PreprocessingRefactoringInfo extends JavaRefactoringInfo {
	
	/**
	 * The preprocessing tag of the selection.
	 */
	private String tag;
	
	/**
	 * Can extract nested tags.
	 */
	private boolean canExtract;

	/**
	 * Creates a description of the selected java code for refactoring.
	 * 
	 * @param offset
	 *            the offset of the selection
	 * @param length
	 *            the length of the selection
	 * @param startLine
	 *            the start line of the selection
	 * @param endLine
	 *            the end line of the selection
	 * @param selectedText
	 *            the selected text
	 * @param origin
	 *            the origin file
	 * @param destinations
	 *            the destination files
	 * @param index
	 *            the index of the selection within a preprocessed block
	 * @param tag
	 *            the preprocessing tag of the current selection
	 * @param ifIsNotUnique
	 * 			  The condition for extract if code selected is just members.	 
	 */
	public PreprocessingRefactoringInfo(int offset, int length, int startLine,
			int endLine, String selectedText, IFile origin,
			String tag, boolean ifIsNotUnique) throws CoreException, IOException {
		super(offset, selectedText.length(), startLine, endLine, selectedText,
				origin);
		this.tag = tag;
		this.canExtract = true;
		this.keepSelection = true;

		this.selectedNodes = PreprocessingUtil
				.createPreprocessingCompilationUnit(origin, offset,
						selectedText.length(), startLine, endLine,
						selectedText);

		this.javaCompilationUnit = PreprocessingUtil.getJavaCompilationUnit();

		super.setStartOffset(PreprocessingUtil.getNewOffset());
		if(!ifIsNotUnique){
			for (ASTNode node : this.selectedNodes) {
				if (!(node instanceof FieldDeclaration || node instanceof MethodDeclaration)) {
					this.canExtract = false;
				}
			}
		}
	}

	/**
	 * Removes a piece of text from a file.
	 * 
	 * @param selectedText
	 *            the selected text
	 * @param source
	 *            the source file
	 * @param startLine
	 *            the start line
	 * @return the file without the selected text
	 */
	public static IFile removeCodeInSource(String selectedText, IFile source,
			int startLine, int offset) throws IOException, CoreException {
		return PreprocessingUtil.removeCodeInSource(selectedText, source,
				startLine, offset);
	}

	/**
	 * Returns the list of nodes in the selection.
	 * 
	 * @return the nodes in the selection
	 */
	@Override
	public LinkedList<ASTNode> getSelectedNodes() {
		return this.selectedNodes;
	}

	/**
	 * The compilation unit of the origin.
	 * 
	 * @return the compilation unit of the origin
	 */
	@Override
	public CompilationUnit getJavaCompilationUnit() {
		return this.javaCompilationUnit;
	}

	/**
	 * Returns the preprocessing tag of the selection.
	 * 
	 * @return the preprocessing tag of the selection
	 */
	public String getTag() {
		return this.tag;
	}

	/*
	 * (non-Javadoc)
	 * @see org.cesar.flip.flipex.JavaRefactoringInfo#generateClone()
	 */
	@Override
	public PreprocessingRefactoringInfo generateClone() {
		PreprocessingRefactoringInfo clone = null;
		try {
			clone =  new PreprocessingRefactoringInfo(this.startOffset,
					this.length, this.startLine, this.endLine,
					this.selectedText, this.origin, this.tag, this.canExtract);
		} catch (CoreException e) {			
		} catch (IOException e) {
		}
		return clone;
	}

	/**
	 * Verifies if the nested tag can be extracted.
	 * @return
	 */
	public boolean isUnique() {
		return this.canExtract;
	}
}
