package org.cesar.flip.flipex;

import org.cesar.flip.flipex.extractors.IExtractionExtraInfo;
import org.eclipse.core.resources.IFile;

/**
 * Encapsulates all the necessary informations for the execution of a
 * refactoring.
 * 
 * @author Fernando Calheiros (fernando.calheiros@cesar.org.br)
 * 
 */
public abstract class RefactoringInfo {

	/**
	 * Offset of the beginning of the text selection
	 */
	protected int startOffset;

	/**
	 * Offset of the start of the text selection
	 */
	protected int endOffset;

	/**
	 * The file from where the code will be extracted
	 */
	protected IFile origin;

	/**
	 * The file where the extracted code will reside
	 */
	protected IFile destination;

	/**
	 * The length of the text selection
	 */
	protected int length;

	/**
	 * The selected text in String format
	 */
	protected String selectedText;

	/**
	 * The start line of the selection
	 */
	protected int startLine;

	/**
	 * The end line of the selection
	 */
	protected int endLine;

	/**
	 * Information generated during extraction. This information is provided by
	 * each extraction developed.
	 */
	private IExtractionExtraInfo extractionExtraInfo;

	/**
	 * The selected code is not deleted from original code.
	 */
	protected boolean keepSelection;

	/**
	 * 
	 * Creates all the information required for performing a code extraction.
	 * 
	 * @param offset
	 *            The offset of the beginning of the text selection
	 * @param length
	 *            The length of the text selection
	 * @param startLine
	 *            The start line of the text selection
	 * @param endLine
	 *            The end line of the text selection
	 * @param selectedText
	 *            The selected text in String format
	 * @param origin
	 *            The file from where the text was selected
	 * @param destinations
	 *            The files where the extracted code will reside
	 */
	public RefactoringInfo(int offset, int length, int startLine, int endLine,
			String selectedText, IFile origin) {
		this.startOffset = offset;
		this.endOffset = offset + length;
		this.length = length;
		this.origin = origin;
		this.selectedText = selectedText;
		this.startLine = startLine;
		this.endLine = endLine;
	}

	/**
	 * Returns the destination of this extraction.
	 * 
	 * @return The destination of the extracted code
	 */
	public IFile getDestination() {
		return this.destination;
	}

	/**
	 * Sets the destination of this extraction.
	 * 
	 * @param destination
	 *            Where the extracted code will reside
	 */
	public void setDestination(IFile destination) {
		this.destination = destination;
	}

	/**
	 * Returns the end offset of the code selection.
	 * 
	 * @return The offset of the end of the text selection
	 */
	public int getEndOffset() {
		return this.endOffset;
	}

	/**
	 * Sets the end offset of the code selection.
	 * 
	 * @param endOffset
	 *            Offset of the end of the text selection
	 */
	public void setEndOffset(int endOffset) {
		this.endOffset = endOffset;
	}

	/**
	 * Returns the length of the code selection.
	 * 
	 * @return The lentgh of the text selection
	 */
	public int getLength() {
		return this.length;
	}

	/**
	 * Sets the length of the code selection.
	 * 
	 * @param lentgh
	 *            The length of the text selection
	 */
	public void setLength(int lentgh) {
		this.length = lentgh;
	}

	/**
	 * Returns the origin file of the selected code.
	 * 
	 * @return The file from where the code will be extracted
	 */
	public IFile getOrigin() {
		return this.origin;
	}

	/**
	 * Sets the origin file of the selected code.
	 * 
	 * @param origin
	 *            The file from where the code will be extracted
	 */
	public void setOrigin(IFile origin) {
		this.origin = origin;
	}

	/**
	 * Returns the selected code for extraction
	 * 
	 * @return The selected text for extraction
	 */
	public String getSelectedText() {
		return this.selectedText;
	}

	/**
	 * Sets the selected code.
	 * 
	 * @param selectedText
	 *            The selected text for extraction
	 */
	public void setSelectedText(String selectedText) {
		this.selectedText = selectedText;
	}

	/**
	 * Returns the start offset of the selection.
	 * 
	 * @return Offset of the start of the text selection
	 */
	public int getStartOffset() {
		return this.startOffset;
	}

	/**
	 * Sets the start offset of the selection.
	 * 
	 * @param startOffset
	 *            Offset of the start of the text selection
	 */
	public void setStartOffset(int startOffset) {
		this.startOffset = startOffset;
	}

	/**
	 * Returns the end line of the selection.
	 * 
	 * @return The end line
	 */
	public int getEndLine() {
		return this.endLine;
	}

	/**
	 * Sets the end line of the selection.
	 * 
	 * @param end
	 *            The end line
	 */
	public void setEndLine(int endLine) {
		this.endLine = endLine;
	}

	/**
	 * Returns the start line of the selection.
	 * 
	 * @return The start line
	 */
	public int getStartLine() {
		return this.startLine;
	}

	/**
	 * Sets the start line of the selection.
	 * 
	 * @param startLine
	 *            The start line
	 */
	public void setStartLine(int startLine) {
		this.startLine = startLine;
	}

	/**
	 * Returns the extraction info.
	 * 
	 * @return The extraction info
	 */
	public IExtractionExtraInfo getExtractionInfo() {
		return this.extractionExtraInfo;
	}

	/**
	 * Sets the extraction info.
	 * 
	 * @param extractionExtraInfo
	 *            The extraction info
	 */
	public void setExtractionInfo(IExtractionExtraInfo extractionInfo) {
		this.extractionExtraInfo = extractionInfo;
	}

	/**
	 * Ask if the selection code must keep in the original class.
	 * @return
	 */
	public boolean isKeepSelection() {
		return this.keepSelection;
	}

	/**
	 * Sets the keepSelection
	 * @param removeSelection
	 */
	public void setKeepSelection(boolean removeSelection) {
		this.keepSelection = removeSelection;
	}

	/**
	 * Generates clones of this refactoringInfo.
	 * @return
	 */
	public abstract RefactoringInfo generateClone();
}
