package org.cesar.flip.flipex.extractors;

import org.cesar.flip.flipex.RefactoringInfo;
import org.eclipse.core.runtime.CoreException;

/**
 * Adapter class for all extractors. <br>
 * <br>
 * Clients who wish to extend the FLIP platform by adding code extractors should
 * subclass this class.
 * 
 * @author Davi Pires (davi.pires@cesar.org.br)
 * 
 */
public abstract class Extractor implements IExtractor {

	/**
	 * The extractor's name.
	 */
	private String name;

	/**
	 * The extractor's description.
	 */
	private String description;

	/**
	 * A description of the requested extraction.
	 */
	protected RefactoringInfo refactoringInfo;

	/**
	 * The technique of extraction employed by this extractor (AOP,
	 * preprocessing, etc.)
	 */
	private IExtractionTechnique technique;

	/**
	 * Initializes the extractor. This method is called immediatly before the
	 * <code>extract()</code>. <br>
	 * <br>
	 * The default implementation stores the <code>info</code> parameter in
	 * the protected field <code>refactoringInfo</code>.<br>
	 * It's highly recommended that extractors which override this method call
	 * its super implementation.
	 * 
	 * @param info
	 *            the <code>RefactoringInfo</code> object to be stored.
	 */
	public void init(RefactoringInfo info) {
		this.refactoringInfo = info;
	}

	/**
	 * @throws CoreException
	 * @see org.cesar.flip.flipex.extractors.IExtractor#extract()
	 */
	public abstract void extract() throws CoreException;

	/**
	 * Clears the internal state of the extractor. This method is called after
	 * the execution of the extraction. It's highly recommended that extractors
	 * which override this method call its super implementation.
	 */
	public void clear() {
		this.refactoringInfo = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cesar.flip.flipex.extractors.IExtractor#getName()
	 */
	public String getName() {
		return this.name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cesar.flip.flipex.extractors.IExtractor#setName(java.lang.String)
	 */
	public void setName(String name) {
		this.name = name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cesar.flip.flipex.extractors.IExtractor#setDescription(java.lang.String)
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cesar.flip.flipex.extractors.IExtractor#getDescription()
	 */
	public String getDescription() {
		return this.description;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cesar.flip.flipex.extractors.IExtractor#getExtractionTechnique()
	 */
	public IExtractionTechnique getExtractionTechnique() {
		return this.technique;
	}

	/*
	 * (non-Javadoc)
	 * @see org.cesar.flip.flipex.extractors.IExtractor#setExtractionTechnique(org.cesar.flip.flipex.extractors.IExtractionTechnique)
	 */
	public void setExtractionTechnique(IExtractionTechnique technique) {
		this.technique = technique;
	}
}
