package org.cesar.flip.flipex.extractors;

import org.cesar.flip.flipex.RefactoringInfo;
import org.eclipse.core.runtime.CoreException;

/**
 * Interface describing what operations FLIPex's extractors must provide.
 * 
 * @author Davi Pires (davi.pires@cesar.org.br)
 * 
 * @see Extractor
 */
public interface IExtractor {

	/**
	 * Initializes the extractor. Allocates needed resources and initializes
	 * attributes that change between each extraction (e.g. initialize a text
	 * buffer, store the name of the destination file).
	 * 
	 * @param info
	 *            the <code>RefactoringInfo</code> object describing the
	 *            extraction
	 */
	public void init(RefactoringInfo info);

	/**
	 * Runs the extraction. Removes the selected code from its origin and places
	 * it into the destination after proper processing (e.g. creating an AspectJ
	 * construct which inserts the code at the point from where it was removed).
	 * 
	 * @throws CoreException
	 */
	public void extract() throws CoreException;

	/**
	 * Clears the internal state of the extractor. Usually, set private fields
	 * to <code>null</code> and releases allocated resources.
	 * 
	 */
	public void clear();

	/**
	 * Sets the name of the extractor.
	 * 
	 * @param attribute
	 *            the name to set.
	 */
	public void setName(String attribute);

	/**
	 * Returns the name of the extractor.
	 * 
	 * @return the name of the extractor.
	 */
	public String getName();

	/**
	 * Sets the extractor's description.
	 * 
	 * @param description
	 *            the description of the extractor.
	 */
	public void setDescription(String description);

	/**
	 * Returns the extractor's description.
	 * 
	 * @return the description of the extractor.
	 */
	public String getDescription();

	/**
	 * The technique of extraction employed by this extractor (AOP,
	 * preprocessing, etc.)
	 * 
	 * @return the extraction technique.
	 */
	public IExtractionTechnique getExtractionTechnique();

	/**
	 * Sets the technique of extraction employed by this extractor (AOP,
	 * preprocessing, etc.)
	 * 
	 * @param technique
	 *            the extraction technique.
	 */
	public void setExtractionTechnique(IExtractionTechnique technique);
}
