package org.cesar.flip.flipex.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.LinkedList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.internal.corext.dom.SelectionAnalyzer;

/**
 * Utility class for dealing with Java files that have preprocessed code in
 * them.
 * 
 * @author Vilmar Nepomuceno (vilmar.nepomuceno@cesar.org.br)
 * 
 */
@SuppressWarnings("restriction")
public class PreprocessingUtil {

	private static int newOffset;

	private static CompilationUnit javaCompilationUnit;

	private static int newLength;

	/**
	 * Creates an AST from the java source code in the given file, with resolved
	 * bindings.
	 * 
	 * @param source
	 *            Java source code file
	 * @param length
	 * @param offset
	 * @return The AST generated from the provided java source code
	 * @throws CoreException
	 * @throws IOException
	 */
	public static LinkedList<ASTNode> createPreprocessingCompilationUnit(
			IFile source, int offset, int length, int startLine, int endLine,
			String selectedText) throws CoreException, IOException {

		String fullPath = source.getLocation().toPortableString();

		PreprocessingUtil.appendSelectedTextToFile(selectedText, source, offset,
				startLine, new File(fullPath));

		source.refreshLocal(IResource.DEPTH_ZERO, null);

		PreprocessingUtil.javaCompilationUnit = PreprocessingUtil
				.createJavaCompilationUnit(source);

		LinkedList<ASTNode> nodes = PreprocessingUtil.getNodes(source,
				PreprocessingUtil.newOffset, newLength,
				PreprocessingUtil.javaCompilationUnit);

		return nodes;
	}

	public static IFile removeCodeInSource(String selectedText, IFile source,
			int startLine, int offset) throws IOException, CoreException {
		String fullPath = source.getLocation().toPortableString();

		int indexOf = fullPath.lastIndexOf(".");
		StringBuffer sb = new StringBuffer(fullPath);
		sb.insert(indexOf, "temp");

		File tempFile = new File(sb.toString());
		tempFile.createNewFile();

		PreprocessingUtil.removeSelectedTextToFile(selectedText, source,
				startLine, new File(fullPath), offset);

		tempFile.delete();
		source.refreshLocal(IResource.DEPTH_ZERO, null);

		return source;
	}

	private static void removeSelectedTextToFile(String selectedText,
			IFile source, int startLine, File tempFile, int offset) throws CoreException,
			IOException {
		BufferedReader bf = new BufferedReader(new InputStreamReader(source
				.getContents()));

		int line = 0;
		StringBuffer newContent = new StringBuffer();
		while (bf.ready()) {
			newContent.append(bf.readLine() + "\n");
			line++;
		}
		bf.close();

		int ofsset = newContent.indexOf(selectedText, offset);

		newContent = newContent.delete(ofsset, ofsset + selectedText.length());

		FileWriter fw = new FileWriter(tempFile);
		PrintWriter pw = new PrintWriter(fw);

		pw.write(newContent.toString());
		pw.flush();
		pw.close();

	}

	private static void appendSelectedTextToFile(String selectedText,
			IFile source, int offset, int startLine, File tempFile) throws CoreException,
			IOException {
		BufferedReader bf = new BufferedReader(new InputStreamReader(source
				.getContents()));

		int line = 0;
		StringBuffer newContent = new StringBuffer();
		while (bf.ready()) {

			if (line == startLine - 1) {
				int actualSize= newContent.length();
				
				for (int i = actualSize; i < offset; i++) {
					newContent.append("\t");
				}
				newContent.append(selectedText.trim());
				String temp = bf.readLine().trim();
				newContent.append(temp + "\n");
				while (!(temp = bf.readLine().trim()).startsWith("//#endif")) {
					if (!temp.startsWith("//#")) {
						newContent.append("\t\t//# " + temp + "\n");
					} else {
						newContent.append("\t\t" + temp + "\n");
					}
				}
				newContent.append("\t\t" +temp.trim()+ "\n");
			}

			newContent.append(bf.readLine() + "\n");
			line++;
		}
		bf.close();
		
		PreprocessingUtil.newOffset = newContent.indexOf(selectedText.trim(), offset);
		PreprocessingUtil.newLength = selectedText.trim().length();
		
		FileWriter fw = new FileWriter(tempFile);
		PrintWriter pw = new PrintWriter(fw);

		pw.write(newContent.toString());
		pw.flush();
		pw.close();
	}

	private static LinkedList<ASTNode> getNodes(IFile source, int offset,
			int length, CompilationUnit javaCompilationUnit) {

		FlipSelection selection = new FlipSelection(offset, length);
		SelectionAnalyzer selectionAnalyzer = new SelectionAnalyzer(selection,
				false);

		javaCompilationUnit.accept(selectionAnalyzer);

		ASTNode nodes[] = selectionAnalyzer.getSelectedNodes();

		LinkedList<ASTNode> nodesList = new LinkedList<ASTNode>();

		for (ASTNode node : nodes) {
			nodesList.add(node);
		}

		return nodesList;
	}

	/**
	 * Creates an AST from the java source code in the given file, with resolved
	 * bindings.
	 * 
	 * @param source
	 *            Java source code file
	 * @return The AST generated from the provided java source code
	 */
	private static CompilationUnit createJavaCompilationUnit(IFile source) {
		ICompilationUnit sourceICompilationUnit = JavaCore
				.createCompilationUnitFrom(source);

		ASTParser parser = ASTParser.newParser(AST.JLS3);

		parser.setSource(sourceICompilationUnit);

		parser.setCompilerOptions(JavaCore.getDefaultOptions());

		parser.setResolveBindings(true);

		CompilationUnit sourceAST = (CompilationUnit) parser.createAST(null);

		sourceAST.recordModifications();

		return sourceAST;
	}

	public static int getNewOffset() {
		return PreprocessingUtil.newOffset;
	}

	public static CompilationUnit getJavaCompilationUnit() {
		return PreprocessingUtil.javaCompilationUnit;
	}
}
