package org.cesar.flip.flipex.validators;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.cesar.flip.flipex.extractors.IExtractor;

/**
 * Encapsulates the results of the validation of the requested transformation.
 * 
 * @author Davi Pires (davi.pires@cesar.org.br)
 * 
 */
public class ValidationResults {

	/**
	 * A map from each extractor to the status of each of its required
	 * validators.
	 */
	private Map<IExtractor, List<ValidatorStatus>> extractorStatusMap;

	/**
	 * Creates a new empty <code>ValidationResults</code> instance.
	 * 
	 */
	public ValidationResults() {
		this.extractorStatusMap = new HashMap<IExtractor, List<ValidatorStatus>>();
	}

	/**
	 * Adds a new <code>ValidatorStatus</code> to the list of failing
	 * validators of the given extractor.
	 * 
	 * This method checks if the given <code>ValidatorStatus</code> actually
	 * failed. If it did not, it returns. Otherwise, we continue to check if the
	 * given extractor is already registered. If it is, we just add
	 * <code>status</code> to its list of failed validators. Otherwise, we
	 * create a new list and <code>put</code> it into the map.
	 * 
	 * @param extractor
	 *            the extractor whose validator failed
	 * @param status
	 *            the status of the validator
	 */
	public void addFailedValidator(IExtractor extractor, ValidatorStatus status) {
		if (status.isSuccessful()) {
			return;
		}

		if (this.extractorStatusMap.containsKey(extractor)) {
			List<ValidatorStatus> statusList = this.extractorStatusMap
					.get(extractor);
			statusList.add(status);
		} else {
			List<ValidatorStatus> statusList = new LinkedList<ValidatorStatus>();
			statusList.add(status);
			this.extractorStatusMap.put(extractor, statusList);
		}
	}

	/**
	 * Returns the list of validators required by the given extractor that
	 * failed the validation.
	 * 
	 * @param extractor
	 *            the extractor whose failed validators status are requested
	 * @return the status of the validators that failed
	 */
	public List<ValidatorStatus> getFailedValidatorsFor(IExtractor extractor) {
		return this.extractorStatusMap.get(extractor);
	}

	/**
	 * Maps an extractor to an empty list of <code>ValidatorStatus</code>.
	 * 
	 * @param extractor
	 *            the extractor that will be added
	 */
	public void addExtractor(IExtractor extractor) {
		this.extractorStatusMap.put(extractor,
				new LinkedList<ValidatorStatus>());
	}

	/**
	 * Returns the extractors whose validators were all sucessful.
	 * 
	 * @return a list of <code>IExtractor</code>s.
	 */
	public List<IExtractor> getSuccessfulExtractors() {
		List<IExtractor> successfulExtractors = new LinkedList<IExtractor>();

		Set<IExtractor> extractors = this.extractorStatusMap.keySet();
		for (IExtractor extractor : extractors) {
			if (this.getFailedValidatorsFor(extractor).size() == 0) {
				successfulExtractors.add(extractor);
			}
		}
		return successfulExtractors;
	}

	/**
	 * Returns the extractors whose validators did not succeed.
	 * 
	 * @return a list of <code>IExtractor</code>s.
	 */
	public List<IExtractor> getFailedExtractors() {
		List<IExtractor> failedExtractors = new LinkedList<IExtractor>();

		Set<IExtractor> extractors = this.extractorStatusMap.keySet();
		for (IExtractor extractor : extractors) {
			if (this.getFailedValidatorsFor(extractor).size() > 0) {
				failedExtractors.add(extractor);
			}
		}
		return failedExtractors;
	}

}
