package org.cesar.flip.flipex.validators;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.cesar.flip.flipex.RefactoringInfo;

/**
 * A set of all validators existing in the system.
 * 
 * @author Davi Pires (davi.pires@cesar.org.br)
 * 
 */
public class ValidatorSet {

	/**
	 * A map from the validators names to their objects.
	 */
	private Map<String, IValidator> validators;

	/**
	 * Creates a new empty instance of <code>ValidatorSet</code>.
	 */
	public ValidatorSet() {
		this.validators = new HashMap<String, IValidator>();
	}

	/**
	 * 
	 * Adds a new element to the set.
	 * 
	 * @param newValidator
	 *            to be added to the set.
	 * @return <b><code>this</code></b>.
	 */
	public ValidatorSet addValidator(IValidator newValidator) {
		this.validators.put(newValidator.getClass().getName(), newValidator);

		return this;
	}

	/**
	 * Returns from the set the validator whose class name is specified in the
	 * parameter.
	 * 
	 * @param name
	 *            the fully-qualified name of the validator class.
	 * @return
	 */
	public IValidator getValidator(String name) {
		return this.validators.get(name);
	}

	/**
	 * Calls the <code>validateOrigin</code> method sequentially in all
	 * validators, and returns a list with all the validators' status.
	 * 
	 * @param refactoringInfo
	 *            the description of the refactoring to be validated.
	 * @return a list of <code>ValidatorStatus</code>
	 */
	public List<ValidatorStatus> getValidatorsStatusOrigin(
			RefactoringInfo refactoringInfo) {
		List<ValidatorStatus> results = new LinkedList<ValidatorStatus>();

		Set<String> keys = this.validators.keySet();
		for (String name : keys) {
			IValidator validator = this.validators.get(name);
			ValidatorStatus validationStatus = validator
					.validateOrigin(refactoringInfo);
			results.add(validationStatus);
		}
		return results;
	}

	/**
	 * Calls the <code>validateDestination</code> method sequentially in the
	 * provided validators, and returns a list with all the validators' status.
	 * 
	 * @param refactoringInfo
	 *            the description of the refactoring to be validated.
	 * @param requiredValidators
	 * @return a list of <code>ValidatorStatus</code>
	 */
	public List<ValidatorStatus> getValidatorsStatusDestination(
			RefactoringInfo refactoringInfo, List<IValidator> requiredValidators) {

		List<ValidatorStatus> results = new LinkedList<ValidatorStatus>();

		for (IValidator validator : requiredValidators) {
			ValidatorStatus validationStatus = validator
					.validateDestination(refactoringInfo);
			results.add(validationStatus);
		}

		return results;
	}
}
