package org.cesar.flip.flipg;

import java.util.List;

import org.cesar.flip.flipg.model.IComponent;
import org.cesar.flip.flipg.model.IFeature;
import org.cesar.flip.flipg.model.IFeature.FeatureType;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;

/**
 * Interface that Feature Model Tools adapters must implement.
 * 
 * Provides all the services needed by FLiP from Feature Model Tools.
 * 
 * @author Fernando Calheiros (fernando.calheiros@cesar.org.br)
 * 
 */
public interface IPLAdapter {

	/**
	 * Returns a list of the features in the feature model in the provided
	 * location.
	 * 
	 * @param featureModelPath
	 *            the path to the feature model
	 * @return the list of teatures
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             </ul>
	 */
	public List<IFeature> getFeatures(String featureModelPath)
			throws CoreException;

	/**
	 * Returns a list of the components that are aspects in the configuration
	 * knowledge in the provided location.
	 * 
	 * @param configurationKnowledgePath
	 *            the path to the configuration knowledge
	 * @return the list of the components
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             </ul>
	 */
	public List<IComponent> getComponents(String configurationKnowledgePath,
			String componentType) throws CoreException;

	/**
	 * Updates the feature model with the changes made by the user during the
	 * extraction.
	 * 
	 * @param features
	 *            the features that were created or updated
	 * @param featureModelPath
	 *            the path to the feature model
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             <li><code>ExceptionConstants.CANT_UPDATE_MODELS</code>
	 *             when an error occurs while updating the model.</li>
	 *             </ul>
	 */
	public void updateFeatureModel(List<IFeature> features,
			String featureModelPath) throws CoreException;

	/**
	 * Updates the configuration knowledge with the changes made by the user
	 * during the extraction.
	 * 
	 * @param components
	 *            the components that were created or updated
	 * @param configurationKnowledgePath
	 *            the path to the configuration knowledge
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_OPEN_MODEL</code> when
	 *             the model can't be opened;</li>
	 *             <li><code>ExceptionConstants.MALFORMED_URL</code> when the
	 *             URL is malformed.</li>
	 *             <li><code>ExceptionConstants.CANT_UPDATE_MODELS</code>
	 *             when an error occurs while updating the model.</li>
	 *             </ul>
	 */
	public void updateConfigurationKnowledge(List<IComponent> components,
			String configurationKnowledgePath) throws CoreException;

	/**
	 * Saves all changes made in the models.
	 * 
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_SAVE_MODELS</code> when
	 *             there was an error saving the models.</li>
	 *             </ul>
	 */
	public void saveModels() throws CoreException;

	/**
	 * Closes all open models.
	 * 
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.CANT_CLOSE_MODELS</code> when
	 *             there was an error closing the models.</li>
	 *             </ul>
	 */
	public void closeModels() throws CoreException;

	/**
	 * 
	 * Creates a feature.
	 * 
	 * @param name
	 *            the name of the feature
	 * @param featureType
	 *            the type of the feature
	 * @param parent
	 *            the parent feature
	 * @return the created feature
	 */
	public IFeature createFeature(String name, FeatureType featureType,
			IFeature parent);

	/**
	 * Creates an object that represents a feature expression from a string.
	 * 
	 * @param expression
	 *            the expression
	 * @return the created expression object
	 * @throws CoreException
	 */
	// public IExpression createExpression(String expression) throws
	// CoreException;
	
	/**
	 * Returns the project nature of the variability management tool.
	 * 
	 * @return the project nature of the variability management tool
	 */
	public String getProductLineNature();

	/**
	 * Copies the default files to the project.
	 * 
	 * @param project
	 *            the project where the files will be copied to
	 * @throws CoreException
	 *             with:
	 *             <ul>
	 *             <li><code>ExceptionConstants.ERROR_COPYING_FILES_TO_PROJECT</code>
	 *             when there was an error copying the default files to a new
	 *             project.</li>
	 *             </ul>
	 */
	public void copyFilesToNewProject(IProject project) throws CoreException;

	/**
	 * Acquires the list of models to be transformed in a given project.
	 * 
	 * @param project
	 *            the project containing the models
	 * @return the list of models
	 */
	public List<String> acquireModelsFromProject(IProject project);
}
